// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Src <string uiname="Source";>;
sampler SrcSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Src);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//source transformation
float4x4 tTex: TEXTUREMATRIX <string uiname="Source Transform";>;

//pin for the Pixelshift
int YPixelShift <string uiname="Y Pixel Shift";> = 300;
int TexHeight <string uiname="Texture Height";> = 600;
float DarkThres <string uiname="Darkness Threshold";> = 0.1f;


//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
    float2 Pixelheight : TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS( float4 PosO  : POSITION,float4 TexCd : TEXCOORD0 )
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);
    
     //you can calculated things here aswell
      //we calculated here the "pixelsize" = 1/texture size
    Out.Pixelheight = 1 /float(TexHeight);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 psAlpha_HorSplitVideo(vs2ps In): COLOR
{
    //source texture lookup
    float4 outColor = (0.0,0.0,0.0,1.0);
    float4 col = tex2D(SrcSamp, In.TexCd);
    // the mask texture lookup
    float Resize = float(In.Pixelheight.x)*float(YPixelShift);
    float2 tempPos = float2(In.TexCd.x,(In.TexCd.y+Resize));

    if (In.TexCd.y < Resize) {
        float4 maskcol = tex2D(SrcSamp, tempPos);
        float luminance = (0.222*maskcol.r + 0.707*maskcol.g + 0.071*maskcol.b);
        outColor = col;
        outColor.a = luminance;
     }
    return outColor;
}

float4 psAlpha_DarkVideo(vs2ps In): COLOR
{
    //source texture lookup
    float4 outColor = tex2D(SrcSamp, In.TexCd);

    float luminance = (0.222*outColor.r + 0.707*outColor.g + 0.071*outColor.b);
    if (luminance<DarkThres) {
        outColor.a  = lerp(0.0f,DarkThres,outColor.a);
    }

    return outColor;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique Alph_ByHorizontalSplitVideo
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 psAlpha_HorSplitVideo();
    }
}
technique Alpha_ByDarknessInVideo
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 psAlpha_DarkVideo();
    }
}


technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (SrcSamp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
